(******* SoL - Open Source Project - http://source.online.free.fr  ***********
This history file header is part of the SoLAccessManager.pas file and
shall be kept with the code. This unit is provided "as is"
with no warranty of any kind, that means you can freely use
it but at your own risk, in respect with the Gnu LGPL license agreement.
(Read the 3S components help file for further information)
=========================================================================
| Version | status  |   date    |        by         |        e mail
-------------------------------------------------------------------------
   0.01     Created    04/2002       Luc DAVID         lucdavid@online.fr
   0.02     Modified   06/2002       Luc DAVID         lucdavid@online.fr
   0.03     Modified   07/2002       Luc DAVID         lucdavid@online.fr
   0.04     Modified   10/2002       Luc DAVID         lucdavid@online.fr
   0.05     Modified   11/2002       Luc DAVID         lucdavid@online.fr
=========================================================================*)
unit SoLAccessManager;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, ActnList, StdCtrls, Registry, ShellApi,SoLSharedMemFile;

(***************************************************************************************
                                TSoLAccessManager
****************************************************************************************)
  Type
  TSoLActionList = Class;

  TSoLActionRecord = record
    Name, Caption, Category : string[100];
    Hint: string[150];
    ImageIndex: Integer;
    end;
   TSoLAccessRecord = record
    Name: string[100];
    Checked, Enabled, Visible, ReadOnly: Boolean;
    ReadLevel, WriteLevel: Byte;
    end;
  PSoLActionRecord = ^TSoLActionRecord;
  PSoLAccessRecord = ^TSoLAccessRecord;
  TApplyMode = (amAutomatic, amManual);

  TSoLUser = class(TPersistent)
  Private
    FSoLUserUID: Integer;
    FSoLUserExtraInfo: String;
    FSoLUserFirstName: String;
    FSoLUserLastName: String;
    FSoLUserShortName: String;
  Public
    property ExtraInfo: String Read FSoLUserExtraInfo;
    property FirstName: String Read FSoLUserFirstName;
    property LastName: String Read FSoLUserLastName;
    property ShortName: String Read FSoLUserShortName;
    property UID: Integer Read FSoLUserUID;
    end;

  TSoLAccessManager = class(TSoLCustomMemDataSharing)
  private
    FSoLAccessList: TList;
    FSoLApplyMode: TApplyMode;
    FSoLUser: TSoLUser;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function SendActionListData: Boolean;
    function GetUserAccess: Boolean;
    function ApplyUserAccess: Boolean ;
    procedure GetFileHeader;overload;
    Procedure SetFileHeader;overload;
    property User: TSoLUser read FSoLUser write FSoLUser;
  published
     property TargetApplication;
     property AfterDataSent;
     property BeforeSendData;
     property OnDataReady;
     property OnDataRequest;
     property ApplyMode: TApplyMode read FSoLApplyMode write FSoLApplyMode;
   end;


(***************************************************************************************
                                TSoLAction
****************************************************************************************)
  TSoLAction = class(TAction)
  private
  FSoLReadOnly: Boolean;
  FSoLReadLevel: Byte;
  FSoLWriteLevel: Byte;
  procedure SetReadLevel(Value: Byte);
  procedure SetWriteLevel(Value: Byte);
published
  property ReadOnly: Boolean read FSoLReadOnly write FSoLReadOnly;
  property ReadLevel: Byte read FSoLReadLevel write SetReadLevel;
  property WriteLevel: Byte read FSoLWriteLevel write SetWriteLevel;
  end;
(***************************************************************************************
                                TSoLActionList
*****************************************************************************************)
TSoLActionList = class(TCustomActionList)
  Private
  FSoLAccessManager: TSoLAccessManager;
  Public
  FEditorHwnd: Hwnd;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  published
    property Images;
    property AccessManager: TSoLAccessManager read FSoLAccessManager write FSoLAccessManager;
    property OnChange;
    property OnExecute;
    property OnUpdate;
  end;

(****************************************************************************************)

implementation

(***************************************************************************************
                                TSoLAccessManager
****************************************************************************************)
constructor TSoLAccessManager.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
TargetApplication.Name := 'SoL Manager';
ApplyMode := amManual;
FSoLUser := TSoLUser.Create;
FSoLAccessList := TList.Create;
end;

destructor TSoLAccessManager.Destroy;
var
i: integer;
SoLRecord: PSoLAccessRecord;
begin
  if assigned(FSoLAccessList)
     then
     for i := 0 to (FSoLAccessList.Count - 1) do
      begin
      SoLRecord := FSoLAccessList.Items[i];
      Dispose(SoLRecord);  // bug bug ??
 end;
FSoLAccessList.Free;
FSoLUser.Free;
inherited Destroy;
end;

Procedure TSoLAccessManager.GetFileHeader;
begin
inherited GetFileHeader;
With FSoLData.Header do
  begin
  UserUID                   := User.UID;
  UserExtraInfo             := User.ExtraInfo;
  UserLastName              := User.LastName;
  UserFirstName             := User.FirstName;
  UserShortName             := User.ShortName;
  end;
end;

Procedure TSoLAccessManager.SetFileHeader;
begin
inherited SetFileHeader;
With FSoLData.Header do
  begin
  User.FSoLUserUID := 1;
  User.FSoLUserExtraInfo := UserExtraInfo;
  User.FSoLUserLastName  := UserLastName;
  User.FSoLUserFirstName := UserFirstName;
  User.FSoLUserShortName := UserShortName;
  end;
end;

function TSoLAccessManager.SendActionListData: Boolean;
var
List: TList;
i,j,k: Integer;
C: TComponent;
S: TSoLActionList;
SoLRecord: PSoLActionRecord;
begin
SetFileHeader;
List := TList.Create;
for i := 0 to Screen.FormCount - 1 do
 for j := 0 to Screen.Forms[i].ComponentCount - 1 do begin
  C := Screen.Forms[i].Components[j];
  if C is TSoLActionList then
   if TSoLActionList(C).AccessManager = Self
      then begin
      S := TSoLActionList(C);
      for k := 0 to S.ActionCount - 1 do
      begin
      New(SoLRecord);
      SoLRecord.Name     := S.Actions[k].Name;
      //Stream '&' from caption
      SoLRecord.Caption  := StringReplace(TSoLAction(S.Actions[k]).Caption, '&', '', [rfReplaceAll]);
      SoLRecord.Category := S.Actions[k].Category;
      SoLRecord.Hint     := TSoLAction(S.Actions[k]).Hint;
      List.Add(SoLRecord);
      end;
      end;
    end;
MemoryFile.RecordSize := SizeOf(SoLRecord^);
Result := SendData(List);
List.Free;
end;

function TSoLAccessManager.GetUserAccess: Boolean;
begin
result := ReadData(FSoLAccessList);
if (ApplyMode = amAutomatic) and result
   then ApplyUserAccess;
end;

Function TSoLAccessManager.ApplyUserAccess: Boolean ;
Var
i,j,k: Integer;
C: TComponent;
S: TSoLActionList;
SoLRecord: PSoLAccessRecord;
begin
Result := True;
Try
New(SoLRecord);
for i := 0 to Screen.FormCount - 1 do
 for j := 0 to Screen.Forms[i].ComponentCount - 1 do begin
  C := Screen.Forms[i].Components[j];
  if C is TSoLActionList then
   if TSoLActionList(C).AccessManager = Self
      then begin
      S := TSoLActionList(C);
      for k := 0 to S.ActionCount - 1 do
      begin
      if S.Actions[k].Name = SoLRecord.Name
           then begin
                TSoLAction(S.Actions[j]).Checked := SoLRecord.Checked;
                TSoLAction(S.Actions[j]).Enabled := SoLRecord.Enabled;
                TSoLAction(S.Actions[j]).Visible := SoLRecord.visible;
                end;
      end;
      end;
  end;
Dispose(SoLRecord);
except
Result := False;
end;
end;

(***************************************************************************************
                                TSoLActionList
****************************************************************************************)
constructor TSoLActionList.Create;
begin
inherited Create(AOwner);
FEditorHwnd := 0;
end;

destructor TSoLActionList.Destroy;
begin
if (ComponentState = [csDesigning]) and (FEditorHwnd <> 0) then
   PostMessage(FEditorHwnd, CM_RELEASE, 0, 0);
inherited Destroy;
end;
(***************************************************************************************
                                TSoLAction
****************************************************************************************)
Procedure TSoLAction.SetReadLevel (Value: Byte);
begin
If FSoLReadLevel <> Value
   then if (Value <= 100)
        then FSoLReadLevel := Value
        else if ComponentState = [csDesigning]
             then Application.MessageBox('Enter a value between 0 and 100', 'Warning...', MB_OK);
end;

Procedure TSoLAction.SetWriteLevel (Value: Byte);
begin
If FSoLWriteLevel <> Value
   then if (Value <= 100)
        then FSoLWriteLevel := Value
        else if ComponentState = [csDesigning]
             then Application.MessageBox('Enter a value between 0 and 100', 'Warning...', MB_OK);
end;

end.

