(******* SoL - Open Source Project - http://source.online.free.fr  ***********
This history file header is part of the SoLSharedMemFile.pas file and
shall be kept with the code. This unit is provided "as is"
with no warranty of any kind, that means you can freely use
it but at your own risk, in respect with the Gnu LGPL license agreement.
(Read the 3S components help file for further information)
=========================================================================
| Version | status  |   date    |        by         |        e mail
-------------------------------------------------------------------------
   0.01     Created    04/2002       Luc DAVID         lucdavid@online.fr
   0.02     Modified   06/2002       Luc DAVID         lucdavid@online.fr
   0.03     Modified   07/2002       Luc DAVID         lucdavid@online.fr
   0.04     Modified   10/2002       Luc DAVID         lucdavid@online.fr
   0.05     Modified   11/2002       Luc DAVID         lucdavid@online.fr
=========================================================================*)


unit SoLSharedMemFile;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, StdCtrls, ShellApi;

(***************************************************************************************
                                TSoLCustomDataSharing
****************************************************************************************)
type
  TSoLTransferMode = (sdSynchronize, sdAppend, sdAppendUpdate, sdCopy, sdDelete);
  TSoLLanguages    = (French, English);
  TSoLFileHeader = Record
    CurrentApplicationExeName: String[255];
    CurrentApplicationExtraInfo: String[255];
    CurrentApplicationTitle: String[100];
    CurrentApplicationUID: String[255];
    CurrentApplicationVersion: String[6];
    MemoryFileExtraInfo: String[255];
    MemoryFileRecordSize: Integer;
    MemoryFileRecordCount: Integer;
    MemoryFileSize: Integer;
    MemoryFileTransferMode: TSoLTransferMode;
    UserExtraInfo: String[255];
    UserFirstName: String[30];
    UserLastName: String[30];
    UserShortName: String[4];
    UserUID: Integer;
    end;

  TSoLFile = Record
    Header: TSoLFileHeader;
    Data: TList;
    end;

  TSoLDataReadyEvent = procedure(Sender: TObject) of object;
  TSoLDataRequestEvent = procedure(Sender: TObject; MsgSender: Hwnd; Param: Integer) of object;
  TSoLBeforeSendDataEvent = procedure(Sender: TObject) of object;
  TSoLDataSentEvent = procedure(Sender: TObject) of object;

  TSoLError = class(TObject)
  Private
  FSoLErrLastError: String;
  FSoLErrLanguage: TSoLLanguages;
  FSoLErrSilent: Boolean;
  Public
  property Language: TSoLLanguages Read FSoLErrLanguage Write FSoLErrLanguage;
  property Silent: Boolean Read FSoLErrSilent Write FSoLErrSilent;
  property LastError: String Read FSoLErrLastError;
  procedure Error(ErrorNum: Word; ErrorText1, ErrorText2: String; Fatal: Boolean);
  procedure ShowLastError;
  end;


  TSoLApplication = class(TPersistent)
  Private
    FSoLAppExtraInfo: String;
    FSoLAppTitle: String;
    FSoLAppExeName: String;
    FSoLAppUID: String;
    FSoLAppVersion: String;
  Public
    property ExeName: String Read FSoLAppExeName Write FSoLAppExeName;
    property Title: String Read FSoLAppTitle Write FSoLAppTitle;
  Published
    Property ExtraInfo: String Read FSoLAppExtraInfo Write FSoLAppExtraInfo;
    property Version: String Read FSoLAppVersion Write FSoLAppVersion;
    property UID: String Read FSoLAppUID Write FSoLAppUID;
    end;

  TSoLTargetApplication = class(TPersistent)
  Private
    FSoLTargetAppHandle : hWND;
    FSoLTargetAppName: String;
    FSoLTargetAppPath: String;
    FSoLTargetAppSmartLink: Boolean;
  Public
     constructor Create;
     property Handle: Hwnd Read FSoLTargetAppHandle;
  Published
     // The target Application Title : (the hint which is shown on the windows task bar
     // when the target application is minimized.
     property Name: String Read FSoLTargetAppName Write FSoLTargetAppName;
     // The full path to access Application.exe (usefull to launch the target application if not already in use).
     property Path: String Read FSoLTargetAppPath Write FSoLTargetAppPath;
     property SmartLink: Boolean Read FSoLTargetAppSmartLink Write FSoLTargetAppSmartLink;
    end;

//  PSoLSharedMemFilePrivateView = ^Cardinal;
  PSoLSharedMemFilePosition = ^Cardinal;
  PSoLSharedMemFileEnd = ^Cardinal;
  TSoLSharedMemFile = class(TPersistent)
  Private
    FSoLSharedMemFileError: TSoLError;
    FSoLSharedMemFileExtraInfo: String;
    FSoLSharedMemFileGlobalHandle: Hwnd;
    FSoLSharedMemFileName: String;
    FSoLSharedMemFileEnd: PSoLSharedMemFileEnd;
    FSoLSharedMemFilePosition: PSoLSharedMemFilePosition;
    FSoLSharedMemFilePrivateView: Pointer;
    FSoLSharedMemFileSize: Cardinal;
    FSoLRecordCapacity: Cardinal;
    FSoLRecordCount: Cardinal;
    FSoLRecordSize: Cardinal;
    FSoLTransferMode: TSoLTransferMode;
    FSoLSharedMemFileActive: Boolean;
    function GetBof: Boolean;
    function GetEof: Boolean;
    function GetExists: Boolean;
    Procedure SetFSoLSharedMemFileName (Value: String);
    Procedure SetFSoLRecordCapacity(Value: Cardinal);
    Procedure SetFSoLRecordSize(Value: Cardinal);
    Public
    constructor Create;
    Destructor Destroy; override;
    property Active: Boolean Read FSoLSharedMemFileActive;
    property Bof: Boolean Read GetBof;
    property Eof: Boolean Read GetEof;
    property Exists: Boolean Read GetExists;
    property GlobalHandle: Hwnd Read FSoLSharedMemFileGlobalHandle;
    property PrivateView: Pointer Read FSoLSharedMemFilePrivateView;
    property Position: PSoLSharedMemFilePosition Read FSoLSharedMemFilePosition;
    property RecordCount: Cardinal Read FSolrecordCount Write FSolrecordCount;
    property RecordSize: Cardinal Read FSoLRecordSize Write SetFSoLRecordSize;
    Property Size: Cardinal Read FSoLSharedMemFileSize Write FSoLSharedMemFileSize;
    function CreateFile: Boolean;
    procedure Clear;
    function Close: Boolean;
    function ClosePrivateView: Boolean;
    procedure Next;
    function Open: Boolean;
    function OpenPrivateView: Boolean;
    procedure Previous;
    function Read(var Buffer; Count: Cardinal): Boolean;
    function ReadFileHeader(var Buffer): Boolean;
    function ReadRecord(var Buffer): Boolean;
    function Seek(Origin: Word; count: LongInt): Pointer;
    function Write(const Buffer; Count: Cardinal): Boolean;
    function WriteFileHeader(const Buffer): Boolean;
    function WriteRecord(const Buffer): Boolean;
  Published
    property Capacity: Cardinal Read FSoLRecordCapacity Write SetFSoLRecordCapacity;
    Property ExtraInfo: String Read FSoLSharedMemFileExtraInfo Write FSoLSharedMemFileExtraInfo;
     // Name of the shared memory file that will be created.
    Property Name: String Read FSoLSharedMemFileName Write SetFSoLSharedMemFileName;
    // Transfer Type (Usually sdSynchronize).
    property TransferMode: TSoLTransferMode Read FSoLTransferMode Write FSoLTransferMode;
    end;

  TSoLOptions = class(TPersistent)
  Private
    FSoLLanguage: TSoLLanguages;
    FSoLSilent: Boolean;
  Published
    property Language: TSoLLanguages Read FSoLLanguage Write FSoLLanguage;
    property Silent: Boolean Read FSoLSilent Write FSoLSilent;
    end;

(***************************************************************************************
                                TSoLCustomMemDataSharing
****************************************************************************************)
  TSoLCustomMemDataSharing = class(TComponent)
  private
    FSoLApplication: TSoLApplication;
    FSoLComponentVersion: String;
    FSoLError: TSoLError;
    FSoLIncomingFileName: String;
    FSoLOptions: TSoLOptions;
    FSoLSharedMemFile: TSoLSharedMemFile;
    FSoLTargetApplication: TSoLTargetApplication;
    FSoLRequestingDataMsgHandle: hWND;
    FSoLSendingDataMsgHandle: hWND;
    // Events
    FSoLAfterDataSent: TSoLDataSentEvent;
    FSoLBeforeSendData: TSoLBeforeSendDataEvent;
    FSoLDataReady: TSoLDataReadyEvent;
    FSoLDataRequest: TSoLDataRequestEvent;
    Procedure SetFSoLComponentVersion(Value: String);
  protected
     FSoLData: TSoLFile;
     property CurrentApplication: TSoLApplication Read FSoLApplication Write FSoLApplication;
     property MemoryFile: TSoLSharedMemFile Read FSoLSharedMemFile Write FSoLSharedMemFile;
     property Options: TSoLOptions Read FSoLOptions Write FSoLOptions;
     Property TargetApplication: TSoLTargetApplication Read FSoLTargetApplication Write FSoLTargetApplication;
     //Methods
     function ReadData(var List: TList): Boolean;
     function SendData(const DataList: TList): Boolean;

     Property SoLRequestingDataMsgHandle: hWND Read FSoLRequestingDataMsgHandle;
     Property SoLSendingDataMsgHandle: hWND Read FSoLSendingDataMsgHandle;

     procedure GetFileHeader;
     procedure SetFileHeader;
   public
    { Dclarations publiques }
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;

     procedure HandleMessages(var Msg: TMsg; var Handled: Boolean);
     procedure RequestData(Param :Integer);

     // Events
     property AfterDataSent: TSoLDataSentEvent read FSoLAfterDataSent write FSoLAfterDataSent;
     property BeforeSendData: TSoLBeforeSendDataEvent read FSoLBeforeSendData write FSoLBeforeSendData;
     property OnDataReady: TSoLDataReadyEvent read FSoLDataReady write FSoLDataReady;
     property OnDataRequest: TSoLDataRequestEvent read FSoLDataRequest write FSoLDataRequest;
   published
     property ComponentVersion: String Read FSoLComponentVersion Write SetFSoLComponentVersion;
  end;

(***************************************************************************************
                                TSoLMemDataSharing
****************************************************************************************)
 TSoLMemDataSharing = class(TSoLCustomMemDataSharing)
  public
     function ReadData(var List: TList): Boolean; overload;
     function SendData(const DataList: TList): Boolean; overload;
  published
     property CurrentApplication;
     property MemoryFile;
     property Options;
     property TargetApplication;
     property AfterDataSent;
     property BeforeSendData;
     property OnDataReady;
     property OnDataRequest;
  end;

(****************************************************************************************)

Const
TSOL_VERSION = '0.05';

FILENAME_RESET = 1000;
FILENAME_ADDCHAR = 1001;
FILENAME_READY = 1002;

ERR_CLOSED_DATASET = 0;
ERR_INVALIDE_FILENAME = 1;
ERR_ACTIVE_DATASET = 2;
ERR_INVALIDE_RECORDSIZE = 3;
ERR_FILE_ALREADYEXISTS = 4;
var
ErrorStr: array[TSoLLanguages,0..99] of String;


implementation


// TSoLError -------------------
procedure TSoLError.Error(ErrorNum: Word; ErrorText1, ErrorText2: String; Fatal: Boolean);
begin
FSoLErrLastError := Format(ErrorStr[Language,ErrorNum] + ' %s %s', [ErrorText1, ErrorText2]);
if not Silent then
   if Fatal then raise Exception.Create(LastError)
            else Application.MessageBox(PChar(LastError), 'Warning...', MB_OK);
end;

procedure TSoLError.ShowLastError;
begin
raise Exception.Create(LastError);
end;

// TSoLTargetApplication -------------------
constructor TSoLTargetApplication.Create;
begin
inherited Create;
SmartLink := True;
end;

// TSoLSharedMemFile -----------------------------------------
constructor TSoLSharedMemFile.Create;
begin
inherited Create;
FSoLSharedMemFilePosition := nil;
FSoLRecordCapacity := 100;
FSoLRecordSize := 0;
FSoLSharedMemFileGlobalHandle := 0;
FSoLSharedMemFileActive := False;
FSoLSharedMemFileError := TSoLError.Create;
Name := 'MemoryFile';
end;

destructor TSoLSharedMemFile.Destroy;
begin
FSoLSharedMemFileError.Free;
inherited Destroy;
end;
//====================================

// Bof
function TSoLSharedMemFile.GetBof: Boolean;
begin
Result := (FSoLSharedMemFilePosition^ = Cardinal(FSoLSharedMemFilePrivateView^));
end;

// Capacity
Procedure TSoLSharedMemFile.SetFSoLRecordCapacity(Value: Cardinal);
begin
If (Value <> FSoLRecordCapacity) and (Value > 0)
   then begin
   FSoLRecordCapacity := Value;
   FSoLSharedMemFileSize := SizeOf(TSoLFileHeader)+(Value*RecordSize);
   end;
end;

//Clear
procedure TSoLSharedMemFile.Clear;
begin
if Active then
begin
System.FillChar(FSoLSharedMemFilePrivateView^, FSoLSharedMemFileSize, $FF);//Clean the memory space
FlushViewOfFile(FSoLSharedMemFilePrivateView,0);
end
end;

//Close
function TSoLSharedMemFile.Close: Boolean;
begin
Result := CloseHandle(FSoLSharedMemFileGlobalHandle);
end;

//ClosePrivateView
function TSoLSharedMemFile.ClosePrivateView: Boolean;
begin
Result := UnmapViewOfFile(FSoLSharedMemFilePrivateView);
FSoLSharedMemFileActive := False;
end;

//CreateFile
function TSoLSharedMemFile.CreateFile: Boolean;
begin
if RecordSize = 0 then FSoLSharedMemFileError.Error(ERR_INVALIDE_RECORDSIZE, '', '', False);

if Open // Does File already exist ?
   then FSoLSharedMemFileError.Error(ERR_FILE_ALREADYEXISTS, '', '', False)
   else FSoLSharedMemFileGlobalHandle := CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE, 0, Size, PChar(Name));
Result := (FSoLSharedMemFileGlobalHandle <> 0);
end;

// Eof
function TSoLSharedMemFile.GetEof: Boolean;
begin
Result := FSoLSharedMemFilePosition = PSoLSharedMemFilePosition(FSoLSharedMemFileEnd);
end;

// Exists
function TSoLSharedMemFile.GetExists: Boolean;
begin
Result := Open;
end;

// Name
Procedure TSoLSharedMemFile.SetFSoLSharedMemFileName (Value: String);
begin
If (Value <> FSoLSharedMemFileName) and (Value <> '') then
   if Active then FSoLSharedMemFileError.Error(ERR_ACTIVE_DATASET, Name, ' [Name] ', False)
             else
             if pos('\', Value) = 0
                then FSoLSharedMemFileName := Value
                else FSoLSharedMemFileError.Error(ERR_INVALIDE_FILENAME, Name, '', False);
end;

//Next
procedure TSoLSharedMemFile.Next;
begin
seek(soFromCurrent, RecordSize);
end;

// RecordSize
Procedure TSoLSharedMemFile.SetFSoLRecordSize(Value: Cardinal);
begin
If (Value <> FSoLRecordSize) and (Value > 0)
   then begin
   FSoLRecordSize := Value;
   FSoLSharedMemFileSize := SizeOf(TSoLFileHeader)+(Capacity*Value);
   end;
end;

//Open
function TSoLSharedMemFile.Open: Boolean;
begin
FSoLSharedMemFileGlobalHandle := OpenFileMapping(FILE_MAP_ALL_ACCESS, False, PChar(Name));
Result := (FSoLSharedMemFileGlobalHandle <> 0);
end;

//OpenPrivateView
function TSoLSharedMemFile.OpenPrivateView: Boolean;
begin
 FSoLSharedMemFilePrivateView := MapViewOfFile(FSoLSharedMemFileGlobalHandle, FILE_MAP_ALL_ACCESS, 0, 0, 0);
 Result := (FSoLSharedMemFilePrivateView <> nil);
 FSoLSharedMemFileActive := Result;
 if Active then
 begin
 FSoLSharedMemFilePosition := FSoLSharedMemFilePrivateView;
 FSoLSharedMemFileEnd := Pointer(Cardinal(FSoLSharedMemFilePrivateView) + FSoLSharedMemFileSize);
 end;
end;

//Previous
procedure TSoLSharedMemFile.Previous;
begin
seek(soFromCurrent, -RecordSize);
end;

//Read
function TSoLSharedMemFile.Read(var Buffer; Count: Cardinal): Boolean;
begin
Result := True;
Try
If Active then System.Move(Pointer(FSoLSharedMemFilePosition^), Buffer, Count);
Except
Result := False;
end;
end;

//ReadFileHeader
function TSoLSharedMemFile.ReadFileHeader(Var Buffer): Boolean;
begin
Result := True;
Try
If Active then
   begin
   System.Move(Pointer(FSoLSharedMemFilePosition^), Buffer, SizeOf(TSoLFileHeader));
   Seek(soFromBeginning, SizeOf(TSoLFileHeader));
   end;
Except
Result := False;
end;
end;

//ReadRecord
function TSoLSharedMemFile.ReadRecord(var Buffer): Boolean;
Begin
Result := Read(Buffer,RecordSize);
end;

//Seek
function TSoLSharedMemFile.Seek(Origin: Word; count: LongInt): pointer;
begin
If Active then
    case Origin of
      soFromBeginning: FSoLSharedMemFilePosition := Pointer(Cardinal(FSoLSharedMemFilePrivateView) + Count);
      soFromCurrent:   FSoLSharedMemFilePosition := Pointer(Cardinal(FSoLSharedMemFilePosition) + Count);
      soFromEnd:       FSoLSharedMemFilePosition := Pointer(Cardinal(FSoLSharedMemFilePosition) + Count);
      end;
    Result := FSoLSharedMemFilePosition;
end;

//Write
function TSoLSharedMemFile.Write(const Buffer; Count:Cardinal): Boolean;
begin
Result := True;
Try
if Active then
begin
  System.Move(Buffer,Pointer(Cardinal(FSoLSharedMemFilePosition))^,Count);
  FlushViewOfFile(FSoLSharedMemFilePrivateView,0);
end;
Except
Result := False;
FSoLSharedMemFileError.Error(ERR_CLOSED_DATASET, Name, ' [Write] ', False);
end;
end;

//WriteFileHeader
function TSoLSharedMemFile.WriteFileHeader(Const Buffer): Boolean;
begin
Result := True;
Try
Write(Buffer,SizeOf(TSoLFileHeader));
Seek(soFromBeginning, SizeOf(TSoLFileHeader))
Except
Result := False;
end;
end;

//WriteRecord
function TSoLSharedMemFile.WriteRecord(Const Buffer): Boolean;
begin
Result := Write(Buffer,RecordSize);
end;


// TSoLCustomMemDataSharing --------------------------------------
constructor TSoLCustomMemDataSharing.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FSoLApplication := TSoLApplication.Create;
FSoLError := TSoLError.Create;
FSoLOptions := TSoLOptions.Create;
FSoLSharedMemFile := TSoLSharedMemFile.Create;
FSoLTargetApplication := TSoLTargetApplication.Create;
FSoLData.Data := TList.Create;
FSoLComponentVersion := TSOL_VERSION;
FSoLSendingDataMsgHandle := registerWindowMessage('WM_SoLSendingData');
FSoLRequestingDataMsgHandle := registerWindowMessage('WM_SoLRequestingData');
MemoryFile.TransferMode := sdSynchronize;
TargetApplication.Name := '';
SetFileHeader;
end;

destructor TSoLCustomMemDataSharing.Destroy;
var
i: integer;
SoLRecord: PByteArray;
begin
  if assigned(FSoLData.Data)
     then begin
     for i := 0 to (FSoLData.Data.Count - 1) do
      begin
      SoLRecord := FSoLData.Data.Items[i];
      Dispose(SoLRecord);
      end;
      FSoLData.Data.Free;
     end;
FSoLApplication.Free;
FSoLError.Free;
FSoLOptions.Free;
FSoLSharedMemFile.Free;
FSoLTargetApplication.Free;
inherited Destroy;
end;
// ------------------------------------------------------------

// ComponentVersion
procedure TSoLCustomMemDataSharing.SetFSoLComponentVersion(Value: String);
begin
// Read Only;
end;

// FileHeader
procedure TSoLCustomMemDataSharing.GetFileHeader;
begin
With FSoLData.Header do
  begin
  CurrentApplication.ExeName   := CurrentApplicationExeName;
  CurrentApplication.ExtraInfo := CurrentApplicationExtraInfo;
  CurrentApplication.Title     := CurrentApplicationTitle;
  CurrentApplication.Version   := CurrentApplicationVersion;
  CurrentApplication.UID       := CurrentApplicationUID;
  MemoryFile.ExtraInfo         := MemoryFileExtraInfo;
  MemoryFile.RecordCount       := MemoryFileRecordCount;
  MemoryFile.RecordSize        := MemoryFileRecordSize;
  MemoryFile.Size              := MemoryFileSize;
  MemoryFile.TransferMode      := MemoryFileTransferMode;
  end;
end;

procedure TSoLCustomMemDataSharing.SetFileHeader;
begin
With FSoLData.Header do
  begin
  CurrentApplicationExeName  := CurrentApplication.ExeName;
  CurrentApplicationExtraInfo:= CurrentApplication.ExtraInfo;
  CurrentApplicationTitle    := CurrentApplication.Title;
  CurrentApplicationVersion  := CurrentApplication.Version;
  CurrentApplicationUID      := CurrentApplication.UID;
  MemoryFileExtraInfo        := MemoryFile.ExtraInfo;
  MemoryFileRecordCount      := MemoryFile.RecordCount;
  MemoryFileRecordSize       := MemoryFile.RecordSize;
  MemoryFileSize             := MemoryFile.Size;
  MemoryFileTransferMode     := MemoryFile.TransferMode;
  end;
end;

// HandleMessages
procedure TSoLCustomMemDataSharing.HandleMessages(var Msg: TMsg; var Handled: Boolean);
begin
If Msg.message = SoLRequestingDataMsgHandle
   then begin
        if assigned(FSoLDataRequest)
           then FSoLDataRequest(self, Msg.wParam, Msg.lParam);
        Handled := True;
        end;

If Msg.message = SoLSendingDataMsgHandle
   then begin
        case Msg.wParam of
        FILENAME_RESET   : FSoLIncomingFileName := '';
        FILENAME_ADDCHAR : FSoLIncomingFileName := FSoLIncomingFileName + Chr(Msg.lParam);
        FILENAME_READY   : begin
                           MemoryFile.Name := FSoLIncomingFileName;
                           if assigned(FSoLDataReady) then FSoLDataReady(self);
                           end;
        end;
        Handled := True;
        end;
end;

// ReadData
function TSoLCustomMemDataSharing.ReadData(var List: TList): Boolean;
var
pRecord: PByteArray;
i: Integer;
begin
Result := True;
Try
Try
List.Clear;
FSoLData.Data.Clear;
MemoryFile.Open;
MemoryFile.OpenPrivateView;
MemoryFile.Seek(soFromBeginning,0);
// Read SolData.Header Content from SharedMemoryFile
MemoryFile.ReadFileHeader(FSoLData.Header);
GetFileHeader;
// Now we have to read data from memoryFile and fill the Tlist object
for i:= 0 to (MemoryFile.RecordCount - 1) do
    begin
    new(pRecord);
    MemoryFile.ReadRecord(pRecord^);
    MemoryFile.Next;
    FSoLData.Data.Add(pRecord);
    List.Add(pRecord);
    end;
Except
Result := False;
end;
finally
MemoryFile.ClosePrivateView;
end;
end;

// RequestData
procedure TSoLCustomMemDataSharing.RequestData(Param: Integer);
var
Msg: String;
begin
TargetApplication.FSoLTargetAppHandle := FindWindow('TApplication',PChar(TargetApplication.Name));
if TargetApplication.Handle <> 0
   then PostMessage(TargetApplication.Handle,SoLRequestingDataMsgHandle,Application.Handle,Param)
   Else begin
        Msg := 'The application : [' + TargetApplication.Name + '] is not active on your system. The notification message has not been sent.';
        Application.MessageBox(PChar(Msg), 'Warning...', MB_OK);
        end;
end;


// SendData
function TSoLCustomMemDataSharing.SendData(Const DataList: TList): Boolean;
var
i: Integer;
zFileName: array[0..250] of Char;
Msg: String;
begin
Result := True;
Try
Try
if assigned(FSoLBeforeSendData) then FSoLBeforeSendData(self);
DataList.Pack;
MemoryFile.RecordCount := Datalist.count;
if Not MemoryFile.Exists
   then MemoryFile.CreateFile;
MemoryFile.OpenPrivateView;
MemoryFile.clear;
SetFileHeader;
MemoryFile.WriteFileHeader(FSoLData.Header);
// Now we have to Write data
for i:= 0 to (DataList.Count - 1) do
    begin
    MemoryFile.WriteRecord(DataList.Items[i]^);
    MemoryFile.Next;
    end;
//  NOTIFICATION MESSAGE
// Windows calls to send a notification message to target application
// warning the application something is ready for reading in memory
// Get application handle  = Application.title of searched main window
TargetApplication.FSoLTargetAppHandle := FindWindow('TApplication',PChar(TargetApplication.Name));
if (TargetApplication.Handle = 0)
    and (TargetApplication.SmartLink = True)
    and (TargetApplication.Path <> '')
    then TargetApplication.FSoLTargetAppHandle := ShellApi.ShellExecute(Application.MainForm.Handle, nil, StrPCopy(zFileName, TargetApplication.Path),'','', SW_SHOWNOACTIVATE); //>= 32

if TargetApplication.Handle <> 0
   then begin
        PostMessage(TargetApplication.Handle,SoLSendingDataMsgHandle,FILENAME_RESET,0);
        For i := 1 to Length(MemoryFile.Name) do
            PostMessage(TargetApplication.Handle,SoLSendingDataMsgHandle,FILENAME_ADDCHAR,Ord(MemoryFile.Name[i]));
        PostMessage(TargetApplication.Handle,SoLSendingDataMsgHandle,FILENAME_READY,0);
        if assigned(FSoLAfterDataSent)
           then FSoLAfterDataSent(self);
        end
   Else begin
        Msg := 'The application : [' + TargetApplication.Name + '] is not active on your system. The notification message has not been sent.';
        Application.MessageBox(PChar(Msg), 'Warning...', MB_OK);
        end;
except
Result := False;
end;
finally
MemoryFile.ClosePrivateView;
end;
end;

// TSoLMemDataSharing -----------------------------------
// ReadData
function TSoLMemDataSharing.ReadData(var List: TList): Boolean;
begin
Result := inherited ReadData(List);
end;
// SendData
function TSoLMemDataSharing.SendData(Const DataList: TList): Boolean;
begin
Result := inherited SendData(DataList);
end;

// ==========================================================================
// Initialization --------------------------
initialization
ErrorStr[French, ERR_CLOSED_DATASET] := 'Opration non autorise sur un ensemble de donnes ferm.';
ErrorStr[English, ERR_CLOSED_DATASET] := 'Invalide operation on a closed dataset.';

ErrorStr[French, ERR_ACTIVE_DATASET] := 'Opration non autorise sur un ensemble de donnes en cours d''utilisation.';
ErrorStr[English, ERR_ACTIVE_DATASET] := 'Invalide operation on an active dataset.';

ErrorStr[French, ERR_INVALIDE_FILENAME] := 'Nom de fichier manquant ou incorrect. Le symbol <\> n''est pas autoris.';
ErrorStr[English, ERR_INVALIDE_FILENAME] := 'Incorrect or missing file name. The symbol <\> is not allowed.';

ErrorStr[French, ERR_INVALIDE_RECORDSIZE] := 'Proprit [RecordSize] : valeur incorrecte (doit tre > 0)';
ErrorStr[English, ERR_INVALIDE_RECORDSIZE] := 'Property [RecordSize] : Incorrect value (must be > 0).';

ErrorStr[French, ERR_FILE_ALREADYEXISTS] := 'Le fichier existe dj en mmoire.';
ErrorStr[English, ERR_FILE_ALREADYEXISTS] := 'File already exists in memory.';

end.


