unit DBFloatEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, DBCtrls, Db;

type
  TCustomDBFloatEdit = class(TCustomMaskEdit)
  private
    FDataLink: TFieldDataLink;
    FCanvas: TControlCanvas;
    FAlignment: TAlignment;
    FFocused: Boolean;

    // ES
    FDecimalPlaces      : Byte;
    FMin,FMax    : Extended;
    FDec         : Char;
    FErrorText   : string;
    FOldVal      : Extended;
    FTempValue   : Extended;
    // ES

    procedure ActiveChange(Sender: TObject);
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    function GetTextMargins: TPoint;
    procedure ResetMaxLength;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetFocused(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure WMUndo(var Message: TMessage); message WM_UNDO;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;

    // ES
    procedure SetAlignment(AValue:TAlignment);
    procedure ShowError;
    function GetText: string;
    // ES
  protected
    procedure Change; override;
    function EditCanModify: Boolean; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Reset; override;

    // ES
    procedure SetValue(AValue : Extended);
    procedure SetMin(AValue : Extended);
    procedure SetMax(AValue : Extended);
    procedure SetDecimalPlaces(AValue : Byte);
    function GetValue : Extended;
    procedure DoExit;override;
    property Value    : Extended read GetValue write SetValue;
    // ES
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    property Field: TField read GetField;
  published
    // ES
    property DecimalPlaces   : Byte read FDecimalPlaces write SetDecimalPlaces;
    property Text: string read GetText;
    property Min : Extended read FMin write SetMin;
    property Max : Extended read FMax write SetMax;
    property ErrorMessage :string read FErrorText write FErrorText;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    // ES

    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TDBFloatEdit = class(TCustomDBFloatEdit)
    property DecimalPlaces   : Byte read FDecimalPlaces write SetDecimalPlaces;
    property Text: string read GetText;
    property Min : Extended read FMin write SetMin;
    property Max : Extended read FMax write SetMax;
    property ErrorMessage :string read FErrorText write FErrorText;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

const
  NO_TEXT = '';


procedure Register;
begin
  RegisterComponents('Samples', [TDBFloatEdit]);
end;

{ TCustomDBFloatEdit }

procedure TCustomDBFloatEdit.ResetMaxLength;
var
  F: TField;
begin
  if (MaxLength > 0) and Assigned(DataSource) and Assigned(DataSource.DataSet) then
  begin
    F := DataSource.DataSet.FindField(DataField);
    if Assigned(F) and (F.DataType in [ftString, ftWideString]) and (F.Size = MaxLength) then
      MaxLength := 0;
  end;
end;

constructor TCustomDBFloatEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited ReadOnly := True;
  ControlStyle := ControlStyle + [csReplicatable];
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
  FDataLink.OnActiveChange := ActiveChange;
  // ES
  FDec := DecimalSeparator;
  FDecimalPlaces := 1;
  FMin := 0;
  FMax := 1E110;
  FErrorText := NO_TEXT;
  inherited Text := FloatToStrF(FMin,ffFixed,18,fDecimalPlaces);
  // ES
end;

destructor TCustomDBFloatEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TCustomDBFloatEdit.Loaded;
begin
  inherited Loaded;
  Value := FTempValue;  // ES
  ResetMaxLength;
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TCustomDBFloatEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TCustomDBFloatEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
    FDataLink.Edit;
end;

procedure TCustomDBFloatEdit.KeyPress(var Key: Char);
var
  ts : string;
  strDec: String;
  i: Integer;
begin
  case Key of
    #10, #13:
      begin
        Key := #0;
        Exit;
      end; {case: #10, #13}
    #32..#255:
      begin
        ts := Copy(Text,1,SelStart)+Copy(Text,SelStart+SelLength+1,MaxInt);
        // If this is not a numeric key, get outta here!
        if not (Key in ['0'..'9',FDec,'-','e']) then
        begin
          inherited;
          Key := #0;
          Exit;
        end;
        // Allow - sign and 'e' scientific notation
        if Key in ['-','e'] then
        begin
          if Pos(Key,ts) <> 0 then
          begin
            inherited;
            Key := #0;
            Exit;
          end;
        end;

        if Key = '-' then
        begin
          if FMin >= 0 then
          begin
            inherited;
            Key := #0;
            Exit;
          end;
        end;

        if Key = FDec then
        begin
          if FDecimalPlaces = 0 then
          begin
            inherited;
            Key := #0;
            Exit;
          end
          else
          begin
            i := Pos(FDec, ts);
            if i > 0 then
            begin
              SelStart :=i;
              SelLength:=FDecimalPlaces;
              Key := #0;
              Exit;
            end;
          end;
        end;
        // replace seltext by keys
        ts := Copy(Text,1,SelStart)+Key+Copy(Text,SelStart+SelLength+1,MaxInt);

        // Check whether more valid worth
        if Pos(FDec,ts) <> 0 then
        begin
          if Length(ts)-Pos(FDec,ts) > FDecimalPlaces then
          begin
            inherited;
            Key := #0;
            Exit;
          end;
        end;

        if Key = '-' then
        begin
          if Pos('-',ts) <> 1 then
          begin
            inherited;
            Key := #0;
            Exit;
          end;
        end;

        if ts = '' then
        begin
          inherited;
          Key := #0;
          inherited Text := FloatToStrF(FMin,ffFixed,18,FDecimalPlaces);
          SelectAll;
          Exit;
        end;

        if ts = '-' then
        begin
          inherited;
          Key := #0;
          strDec := '';
          for i := 1 to FDecimalPlaces do
            strDec := strDec + '0';
          inherited Text := '-0'+FDec+strDec;
          SelStart :=1;
          SelLength:=FDecimalPlaces+2;
        end;

        if ts = FDec then
        begin
          inherited;
          Key := #0;
          strDec := '';
          for i := 1 to FDecimalPlaces do
            strDec := strDec + '0';
          inherited Text := '0'+FDec+strDec;
          SelStart :=2;
          SelLength:=FDecimalPlaces;
          Exit;
        end;
        inherited;

        FDataLink.Edit;
      end; {case: ^H, ^V, ^X, #32..#255}
    #27:
      begin
        Value := fOldVal;
        SelectAll;
        FDataLink.Reset;
        Key := #0;
        Exit;
      end; {case:#27}
    #8 {BS}, #22 {^V}, #24 {^X}:
      begin
        // Handle Backspace, Copy and Cut.
        // Just set the datalink into editmode and it handles
        // it from there.
        FDataLink.Edit;
      end;
  end; {case}
end;

function TCustomDBFloatEdit.EditCanModify: Boolean;
begin
  Result := FDataLink.Edit;
end;

procedure TCustomDBFloatEdit.SetAlignment(AValue:TAlignment);
begin
  if FAlignment<>AValue then
  begin
    FAlignment:=AValue;
    RecreateWnd;
  end;
end;

procedure TCustomDBFloatEdit.Reset;
begin
  FDataLink.Reset;
  SelectAll;
end;

procedure TCustomDBFloatEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then
  begin
    FFocused := Value;
    if (FAlignment <> taLeftJustify) and not IsMasked then Invalidate;
    FDataLink.Reset;
  end;
end;

procedure TCustomDBFloatEdit.Change;
begin
  FDataLink.Modified;
  inherited Change;
end;

function TCustomDBFloatEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TCustomDBFloatEdit.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TCustomDBFloatEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TCustomDBFloatEdit.SetDataField(const Value: string);
begin
  if not (csDesigning in ComponentState) then
    ResetMaxLength;
  FDataLink.FieldName := Value;
end;

function TCustomDBFloatEdit.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TCustomDBFloatEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

function TCustomDBFloatEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TCustomDBFloatEdit.ActiveChange(Sender: TObject);
begin
  ResetMaxLength;
end;

procedure TCustomDBFloatEdit.DataChange(Sender: TObject);
begin
  // ES
  if FDataLink.Field = nil then
    Value := 0.00
  else
    Value := FDataLink.Field.AsFloat;
  // ES
end;

procedure TCustomDBFloatEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not FDataLink.Editing;
end;

procedure TCustomDBFloatEdit.UpdateData(Sender: TObject);
begin
  ValidateEdit;
  // FDataLink.Field.Text := Text;
  FDataLink.Field.AsFloat := Value;
end;

procedure TCustomDBFloatEdit.WMUndo(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TCustomDBFloatEdit.WMPaste(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TCustomDBFloatEdit.WMCut(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TCustomDBFloatEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  inherited;
  if SysLocale.FarEast and FDataLink.CanModify then
    inherited ReadOnly := False;
end;

procedure TCustomDBFloatEdit.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused(False);
  CheckCursor;
  DoExit;
end;

procedure TCustomDBFloatEdit.DoExit;
var
  ts : string;
  Res: Extended;
begin
  ts := Text;
  if ts = ''
  then
    ts := FloatToStrF(FMin,ffFixed,18,fDecimalPlaces);
  try
    Res := StrToFloat(ts);
    if (Res < FMin) or (Res > FMax)
    then
      Raise Exception.Create(FErrorText);
  except
    ShowError;
    Value := FOldVal;
    SelectAll;
    SetFocus;
    Exit;
  end;
  inherited Text := FloatToStrF(Res,ffFixed,18,FDecimalPlaces);
  Value:= StrToFloat(Text);
  inherited;
end;

procedure TCustomDBFloatEdit.WMPaint(var Message: TWMPaint);
const
  AlignStyle : array[Boolean, TAlignment] of DWORD =
   ((WS_EX_LEFT, WS_EX_RIGHT, WS_EX_LEFT),
    (WS_EX_RIGHT, WS_EX_LEFT, WS_EX_LEFT));
var
  Left: Integer;
  Margins: TPoint;
  R: TRect;
  DC: HDC;
  PS: TPaintStruct;
  S: string;
  AAlignment: TAlignment;
  ExStyle: DWORD;
begin
  AAlignment := FAlignment;
  if UseRightToLeftAlignment then ChangeBiDiModeAlignment(AAlignment);
  if ((AAlignment = taLeftJustify) or FFocused) and
    not (csPaintCopy in ControlState) then
  begin
    if SysLocale.MiddleEast and HandleAllocated and (IsRightToLeft) then
    begin { This keeps the right aligned text, right aligned }
      ExStyle := DWORD(GetWindowLong(Handle, GWL_EXSTYLE)) and (not WS_EX_RIGHT) and
        (not WS_EX_RTLREADING) and (not WS_EX_LEFTSCROLLBAR);
      if UseRightToLeftReading then ExStyle := ExStyle or WS_EX_RTLREADING;
      if UseRightToLeftScrollbar then ExStyle := ExStyle or WS_EX_LEFTSCROLLBAR;
      ExStyle := ExStyle or
        AlignStyle[UseRightToLeftAlignment, AAlignment];
      if DWORD(GetWindowLong(Handle, GWL_EXSTYLE)) <> ExStyle then
        SetWindowLong(Handle, GWL_EXSTYLE, ExStyle);
    end;
    inherited;
    Exit;
  end;
{ Since edit controls do not handle justification unless multi-line (and
  then only poorly) we will draw right and center justify manually unless
  the edit has the focus. }
  if FCanvas = nil then
  begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then DC := BeginPaint(Handle, PS);
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    with FCanvas do
    begin
      R := ClientRect;
      if not (NewStyleControls and Ctl3D) and (BorderStyle = bsSingle) then
      begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
        InflateRect(R, -1, -1);
      end;
      Brush.Color := Color;
      if not Enabled then
        Font.Color := clGrayText;
      if (csPaintCopy in ControlState) and (FDataLink.Field <> nil) then
      begin
        S := FDataLink.Field.DisplayText;
        case CharCase of
          ecUpperCase: S := AnsiUpperCase(S);
          ecLowerCase: S := AnsiLowerCase(S);
        end;
      end else
        S := EditText;
      if PasswordChar <> #0 then FillChar(S[1], Length(S), PasswordChar);
      Margins := GetTextMargins;
      case AAlignment of
        taLeftJustify: Left := Margins.X;
        taRightJustify: Left := ClientWidth - TextWidth(S) - Margins.X - 1;
      else
        Left := (ClientWidth - TextWidth(S)) div 2;
      end;
      if SysLocale.MiddleEast then UpdateTextFlags;
      TextRect(R, Left, Margins.Y, S);
    end;
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0 then EndPaint(Handle, PS);
  end;
end;

procedure TCustomDBFloatEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

function TCustomDBFloatEdit.GetTextMargins: TPoint;
var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
begin
  if NewStyleControls then
  begin
    if BorderStyle = bsNone then I := 0 else
      if Ctl3D then I := 1 else I := 2;
    Result.X := SendMessage(Handle, EM_GETMARGINS, 0, 0) and $0000FFFF + I;
    Result.Y := I;
  end else
  begin
    if BorderStyle = bsNone then I := 0 else
    begin
      DC := GetDC(0);
      GetTextMetrics(DC, SysMetrics);
      SaveFont := SelectObject(DC, Font.Handle);
      GetTextMetrics(DC, Metrics);
      SelectObject(DC, SaveFont);
      ReleaseDC(0, DC);
      I := SysMetrics.tmHeight;
      if I > Metrics.tmHeight then I := Metrics.tmHeight;
      I := I div 4;
    end;
    Result.X := I;
    Result.Y := I;
  end;
end;

function TCustomDBFloatEdit.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TCustomDBFloatEdit.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

procedure TCustomDBFloatEdit.SetValue(AValue : Extended);
var
  tmp : string;
begin
  if csLoading in ComponentState
  then
    FTempValue := AValue
  else
  begin
    if (AValue > FMax) then
    begin
      ShowError;
      AValue := FMax;
    end;
    if (AValue < FMin) then
    begin
      ShowError;
      AValue := FMin;
    end;
    tmp := FloatToStrF(AValue,ffFixed,18,FDecimalPlaces);
    inherited Text:=tmp;
  end;
end;

function TCustomDBFloatEdit.GetValue : Extended;
var
  ts : string;
begin
  if csLoading in ComponentState
  then
    Result := FTempValue
  else
  begin
    ts := text;
    if (ts = '-') or (ts = FDec) or (ts = '') or (ts = 'E') then ts := '0';
    try
       ts := FloatToStrF(StrToFloat(ts), ffFixed,18,FDecimalPlaces);
       Result := StrToFloat(ts);
    except
      Result := FMin;
    end;
    if Result < FMin
    then
       Result := FMin
    else
    if Result > FMax
    then
       Result := FMax
  end;
end;

procedure TCustomDBFloatEdit.SetDecimalPlaces(AValue : Byte);
begin
  if FDecimalPlaces <> AValue then
  begin
    if AValue > 18
    then
      AValue := 18;
    FDecimalPlaces := AValue;
    if FDataLink.Field = nil then
      Value := 0.00
    else
      Value := FDataLink.Field.AsFloat;
  end;
end;

procedure TCustomDBFloatEdit.SetMin(AValue : Extended);
begin
  if FMin <> AValue then
  begin
    if AValue > FMax
    then
      raise Exception.Create('Minimum value must not be greater than maximum value !');
    FMin := AValue;
    Value := Value;
  end;
end;

procedure TCustomDBFloatEdit.SetMax(AValue : Extended);
begin
  if FMax <> AValue then
  begin
    if AValue < FMin
    then
      raise Exception.Create('Maximum value must not be less than minimum value !');
    FMax := AValue;
    Value := Value;
  end;
end;

procedure TCustomDBFloatEdit.ShowError;
begin
  if FErrorText <> NO_TEXT
  then
    ShowMessage(FErrorText);
end;

function TCustomDBFloatEdit.GetText: string;
begin
  Result := inherited Text;
end;

end.
