(**************************************************************************)
(*        TDirDialog - Select folder dialog (version 1.2.0)               *)
(*        Copyright (c) 2003 Maslov Vjacheslav (cooluser@ngs.ru)          *)
(*    Purpose:                                                            *)
(*      Display standard windows "select folder" dialog, located in       *)
(*      shell32.dll,allows user to control input by using very flexible   *)
(*      set of events and change view style (old or new)                  *)
(**************************************************************************)


unit DirDialog;

interface

uses
  Windows, SysUtils, Classes, Controls, shellapi,shlobj,dsgnintf,TypInfo;

const
     Version:='1.2.0';

type
  (*
        Class of property editor About window
  *)
   TAboutProperty = class(TPropertyEditor)
        public
                function GetValue:String;override;
                procedure Edit;override;
                function GetAttributes:TPropertyAttributes;override;
   end;

   (*
      Provide "Test Dialog" popup menu item
   *)
   TDirDialogEditor = class(TComponentEditor)
      function GetVerbCount:integer;override;
      function GetVerb(index:integer):string;override;
      procedure ExecuteVerb(index:integer);override;
   end;

  (*
      Type of some options (properties)
  *)
  TDirDialogOptions= set of (ddoNewStyle,ddoEditBox);
  TInitDir = (idDesktop,idDrives,idPersonal);

  (*
      Events types declaration
  *)
  TBeforeExecute = procedure(var NewPromt:string) of object;
  TOnSelectDirectory = procedure(Name:string;var OkEnabled:boolean) of object;
  TOnInvalidDirectory = procedure (Name:String;var CanClose:boolean) of object;
  TOnShowDialog = procedure(var SelectedName:string) of object;

  (*
      Component class declaration
  *)
  TDirDialog = class(TComponent)
  private
        CallbackReturnValue:integer; //needs to return value in windows API CALLBACK function
        FPromt:string;
        FDirectory:string;
        FBeforeExecute:TBeforeExecute;
        FAbout:TAboutProperty;
        FOptions:TDirDialogOptions;
        Flags:integer;
        FInitDir:TInitDir;
        FOnSelectDirectory:TOnSelectDirectory;
        FOnInvalidDirectory:TOnInvalidDirectory;
        FOnShowDialog:TOnShowDialog;
        function SelectDir:string;
        procedure SetOptions(NewOptions:TDirDialogOptions);
        procedure BrowseCallback(Wnd:HWND;uMsg:UINT;lParam:LongInt); //CALLBACK
        function GetDirCSIDL:integer;
  public
        function Execute:boolean;         //Brings up a dialog
        constructor Create(AOwner:TComponent);override;
  published
        property Directory:string read FDirectory write FDirectory; //Selected directory
        property Promt:string read FPromt write FPromt;             //Text label of top dialog window
        property About:TAboutProperty read FAbout;                  //Just about property
        property Options:TDirDialogOptions read FOptions write SetOptions;
        property InitialDirectory:TInitDir read FInitDir write FInitDir;
        //Events handler declaration
        {This event occurs before dialog displayed and allows to change promt,
        which passed as var parametr}
        property BeforeExecute:TBeforeExecute read FBeforeExecute write FBeforeExecute;

        {
            This event occurs when user select directory (usually just click on it into dialog),
            and allows to change OK button status depending on the name of selected directory)
        }
        property OnSelectDirectory:TOnSelectDirectory read FOnSelectDirectory write FOnSelectDirectory;

        {
          Ocurrs when user type in edit box invalid name of directory
        }
        property OnInvalidDirectory:TOnInvalidDirectory read FOnInvalidDirectory write FOnInvalidDirectory;

        {
          Occurs before dialog brings up on screen, and allows to change initial folder by using
          SelectedName param.
        }
        property OnShowDialog:TOnShowDialog read FOnShowDialog write FOnShowDialog;
  end;

procedure Register;

const
        {Some constants for Shell32.dll version 5, not included in header file (Delphi 5)}
        BIF_NEWDIALOGSTYLE=$40;

implementation

{******************************TDirDialogEditor&&&&&****************************************}
function TDirDialogEditor.GetVerbCount:integer;
begin
      Result:=1;
end;

function TDirDialogEditor.GetVerb(index:integer):string;
begin
      Result:='Test Dialog';
end;
procedure TDirDialogEditor.ExecuteVerb(index:integer);
begin
      (Component as TDirDialog).Execute;
end;

{******************************TAboutProperty************************************************}
function TAboutProperty.GetAttributes:TPropertyAttributes;
begin
        result:=[paDialog,paReadOnly];
end;

function TAboutProperty.GetValue:string;
begin
        Result:='About this component';
end;

procedure TAboutProperty.Edit;
begin
      MessageBox(0,pchar('Copyright (c) 2003 Maslov Vjacheslav (cooluser@ngs.ru) '+Version),pchar('  TDirDialog'),MB_OK or MB_ICONINFORMATION);
end;

{******************************TDirDlg***********************************************}
procedure TDirDialog.SetOptions(NewOptions:TDirDialogOptions);
begin
        if FOptions<>NewOptions then begin
                FOptions:=NewOptions;
                Flags:=0;
                if ddoNewStyle in Options then
                        Flags:=Flags or BIF_NEWDIALOGSTYLE;
                if ddoEditBox in Options then
                        Flags:=Flags or BIF_EDITBOX or BIF_VALIDATE;
        end;
end;

constructor TDirDialog.Create(AOwner:TComponent);
begin
        if not (AOwner is TWinControl) then
            raise Exception.Create('This component should be placed on TWinControl descendant.');
        inherited Create(AOwner);
        FPromt:='Select directory';
        Options:=[ddoNewStyle];
end;

procedure TDirDialog.BrowseCallBack(Wnd:HWND;uMsg:UINT;lParam:LongInt);
var
      Dir:string;
      bParam:boolean;
      buf:array[0..max_path-1] of char;
begin
      if uMsg=BFFM_SELCHANGED then begin
            SHGetPathFromIDList(PItemIDList(lParam),buf);
            if Assigned(FOnSelectDirectory) then begin
                  Dir:=Buf;
                  bParam:=True;
                  FOnSelectDirectory(Dir,bParam);
                  SendMessage(Wnd,BFFM_ENABLEOK,0,integer(bParam));
            end;
            CallbackReturnValue:=0;
      end
      else if uMsg=BFFM_INITIALIZED then begin
            if Assigned(FOnShowDialog) then begin
                  FOnShowDialog(Dir);
                  SendMessage(Wnd,BFFM_SETSELECTION,1,integer(pchar(Dir)));
            end; //if assigned
            CallbackReturnValue:=0;
      end
      else if uMsg=BFFM_VALIDATEFAILED then begin
            if Assigned(FOnInvalidDirectory) then begin
                  Dir:=PChar(lParam);
                  FOnInvalidDirectory(Dir,bParam);
                  if bParam then
                        CallbackReturnValue:=0
                  else
                        CallbackReturnValue:=1;
            end; //if assigned
      end;
end; //proc

function Browse(Wnd:HWND;uMsg:UINT;lParam:LongInt;lpData:LongInt):integer stdcall;
begin
      TDirDialog(lpData).BrowseCallBack(Wnd,uMsg,lParam);
      Result:=TDirDialog(lpData).CallbackReturnValue;
end;

//TInitDir = (idDesktop,idDrives,idPersonal);
function TDirDialog.GetDirCSIDL:integer;
begin
      case FInitDir of
            idDesktop:Result:=CSIDL_DESKTOP;
            idDrives:Result:=CSIDL_DRIVES;
            idPersonal:Result:=CSIDL_PERSONAL;
      else
            Result:=0;
      end;
end;

function TDirDialog.SelectDir:string;
var
      BI:_browseInfoA;
      StartPIDL,ResPIDL:PItemIDList;
      S,Path:array[0..max_path-1] of WideChar;
      APromt:string;
begin
      SHGetSpecialFolderLocation((Owner as TWinControl).Handle,GetDirCSIDL,StartPIDL);
      with BI do begin
            hwndOwner:=(OWner as TWinControl).Handle;
            pszDisplayName:=@S;
            lpszTitle:=pchar(Promt);
            ulFlags:=Flags;
            pidlRoot:=StartPidl;
            lpfn:=Browse;
            lParam:=Integer(Self);
            iImage:=1;
      end;

      if Assigned(FBeforeExecute) then begin
              FBeforeExecute(APromt);
              BI.lpszTitle:=pchar(APromt);
      end;


      ResPIDL:=SHBrowseForFolder(TBrowseInfo(BI));
      if ResPIDL<>nil then begin
            SHGetPathFromIDList(ResPIDL,@Path[0]);
            Result:=StrPas(@Path[0]);
      end
      else
            Result:='';
end;

function TDirDialog.Execute:boolean;
var
      Dir:String;
begin
      Dir:=SelectDir;
      if Dir='' then
            Result:=False
      else begin
            Result:=True;
            Directory:=Dir;
      end;
end;

{Common procedures}
procedure Register;
begin
  RegisterComponents('Samples', [TDirDialog]);
  RegisterPropertyEditor(TypeInfo(TAboutProperty),TDirDialog,'About',TAboutProperty);
  RegisterComponentEditor(TDirDialog,TDirDialogEditor);
end;

{end of source file}
end.
